/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 6.4 (Simple Ratio-of-Uniforms method, p. 138)
 *
 * this file: algo06_04.c
 *
 * For a demo how to use this algorithm see the file "demo_algo06_04.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

static double ul, ur, vm, m, (*pdfh)(double);

int setup06_04( double (*pdf)(double), double Af, double mi, double Fm)
/*
 * Algorithm 6.4 (Simple Ratio-of-Uniforms method, p. 138)
 *
 * pdf ... function pointer to the quasi density
 * Af ... area below the quasi density
 * mi ... mode of the density
 * Fm ... CDF value at the mode m; if unknown -1. must be used
 */
{     double um;
      pdfh = pdf;
      m = mi;
/*1*/ vm = sqrt((*pdf)(m));
      um = Af/vm;
/*2*/ if(Fm >= 0.){/* the cdf value at the mode is known */
/*3*/    ul = - Fm*um;
         ur = um+ul;
      }
/*4*/ else{
/*5*/    ul = -um;
         ur = um;
      }
      printf("Setup06_04:\n ul %f ur %f vm %f\n\n",ul,ur,vm);
      return 1;
} 

double generator06_04()
/* is called after the setup to generate random variates */
{ double u,v,x;

/*6*/ while(1){
/*7*/   u = ul+(ur-ul)*uniform();
        v = vm*uniform();
/*8*/   x = u/v + m;
/*9*/   if(v*v<=(*pdfh)(x))
/*10*/    return x;
      }
}
