/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 12.7 (Zero-Maximal-Bivariate), p. 331 
 *
 * this file: algo12_06.c
 *
 * For a demo how to use this algorithm see the file "demo_algo12_06.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"
#include "algo.h"

static double mt1,mt2,mt3,mt4,s;

int setup12_06(double m1,double m2,double m3,double m4)
/*
 * Algorithm 12.6 (4 Moments continuous), p. 326 
 * with mixture distribution s*Z+1
 *
 * m1, m2, m3, m4 .... array holding the first four moments 
 */
{
       s = 1.;
       if(!setup12_05(m1,m2,m3,m4)){
         /* m1 m2 m3 m4 not feasible for any mixture distribution */
         printf("ERROR setup12_06: Moments not feasible\nABORTING");
         exit(1);
       }
/*2*/  do{
         s = s*0.5;
/*1*/	 mt1 = m1;
	 mt2 = m2/(1.+s*s);
	 mt3 = m3/(1.+3.*s*s);
	 mt4 = m4/(1.+6.*s*s+3.*s*s*s*s);
       }while(!setup12_05(mt1,mt2,mt3,mt4));
       return 1;
}

double generator12_06()
/* generates a continuous random variate with the desired moments */
{
       double u,y;
/*3*/ y = generator12_05();
/*4*/ u = 1+s*generator02_11();
/*5*/ return y*u;
}


