/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Example 2.22, p. 36, 
 * Half normal distribution by almost-exact inversion
 *
 * this file: example02_22.c
 *
 * to compile this file you can use the command:
 *
 * cc -o example02_22 example02_22.c uniform.c statutil.c -lm
 *
 */


#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "statutil.h"
#include "uniform.h"

/**********************************************************************/

double example02_22(void)
/*
 * generates random variates from the half normal distribution 
 * using almost-exact inversion
 */
{ double u, y, x;

/*1*/  while(1){ 
/*2*/    u = uniform();
/*3*/    x = 1.22*u*(1+0.14/(1.-u));
/*4*/    y = uniform();
/*5*/    if(y<=0.8543034*exp(-x*x*0.5)*(1+0.14/((1.-u)*(1.-u)))) 
/*6*/      return x;
       }
}

/*****************************************************************/


#define CLAS 500

static long int obs[CLAS*CLAS];

int main()
{ int i;
 double x;

 setseed(13324885); /*set seed for uniform generator*/

 printf("Example 2.22:\n\nfirst we display ten generated numbers\n");

 for(i=0;i<10;i++){
   x = example02_22();
   printf("x %f\n",x);
 }
 
 printf("\nNow we make a chi-square test with sample size %d\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   x = example02_22();
   obs[(int)(cdf_halfnorm(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}






